package com.datacloudsec.utils;

import com.datacloudsec.scan.common.Constant;
import com.sbrinfo.jsm4sdk.SM4JSDK;
import common.Logger;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.Field;
import java.security.DigestInputStream;
import java.security.Key;
import java.security.MessageDigest;
import java.util.zip.InflaterOutputStream;
import javax.crypto.Cipher;
import javax.crypto.CipherInputStream;
import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import sun.misc.BASE64Decoder;

public class SecretUtil {

   public static final String TYPE_AES = "aes";
   public static final String TYPE_DES = "des";
   public static final String HMAC_MD5 = "HmacMD5";
   public static final String HMAC_SHA1 = "HmacSha1";
   public static final String HMAC_SHA256 = "HmacSha256";
   public static final String SHA256 = "sha-256";
   public static final String SHA1 = "sha-1";
   public static final String SHA512 = "sha-512";
   public static final String SHA384 = "sha-384";
   private static final Logger LOG = Logger.getLogger(SecretUtil.class);


   public static String digestSha1(String str, String type) throws Exception {
      MessageDigest md = MessageDigest.getInstance(type);
      md.update(str.getBytes());
      return toHexString(md.digest());
   }

   public static byte[] digestSha1Byte(String str, String type) throws Exception {
      MessageDigest md = MessageDigest.getInstance(type);
      md.update(str.getBytes());
      return md.digest();
   }

   public static String digestFile(File file, String type) throws Exception {
      FileInputStream is = null;
      DigestInputStream dis = null;
      MessageDigest md = MessageDigest.getInstance(type);

      try {
         is = new FileInputStream(file);
         dis = new DigestInputStream(is, md);
         byte[] buf = new byte[1024];

         while(dis.read(buf) != -1) {
            ;
         }
      } finally {
         IOUtils.closeQuietly(dis);
         IOUtils.closeQuietly(is);
      }

      return toHexString(md.digest());
   }

   public static byte[] symmetryEncrypt(String type, byte[] content, byte[] keys) throws Exception {
      Cipher encryptCipher = Cipher.getInstance(type);
      encryptCipher.init(1, getKey(keys, type));
      return encryptCipher.doFinal(content);
   }

   public static byte[] decryptEncrypt(String type, byte[] content, byte[] keys) throws Exception {
      Cipher decryptCipher = Cipher.getInstance(type);
      decryptCipher.init(2, getKey(keys, type));
      return decryptCipher.doFinal(content);
   }

   public static void symmetryEncryptFile(File file, File destFile, String type, String keys) throws Exception {
      FileInputStream is = new FileInputStream(file);
      FileOutputStream out = new FileOutputStream(destFile);
      Cipher cipherEncrypt = Cipher.getInstance(type);
      cipherEncrypt.init(1, getKey(keys.getBytes("GBK"), type));
      CipherInputStream cis = new CipherInputStream(is, cipherEncrypt);
      byte[] buffer = new byte[1024];

      int r;
      while((r = cis.read(buffer)) > 0) {
         out.write(buffer, 0, r);
      }

      cis.close();
      is.close();
      out.close();
   }

   public static void decryptEncryptFile(File file, File destFile, String type, String keys) throws Exception {
      FileInputStream is = new FileInputStream(file);
      FileOutputStream out = new FileOutputStream(destFile);
      Cipher cipherEncrypt = Cipher.getInstance(type);
      cipherEncrypt.init(2, getKey(keys.getBytes(), type));
      CipherInputStream cis = new CipherInputStream(is, cipherEncrypt);
      byte[] buffer = new byte[1024];

      int r;
      while((r = cis.read(buffer)) > 0) {
         out.write(buffer, 0, r);
      }

      cis.close();
      is.close();
      out.close();
   }

   public static byte[] rc4Base(byte[] input, String mKkey) {
      int x = 0;
      int y = 0;
      byte[] key = initKey(mKkey);
      byte[] result = new byte[input.length];

      for(int i = 0; i < input.length; ++i) {
         x = x + 1 & 255;
         y = (key[x] & 255) + y & 255;
         byte tmp = key[x];
         key[x] = key[y];
         key[y] = tmp;
         int xorIndex = (key[x] & 255) + (key[y] & 255) & 255;
         result[i] = (byte)(input[i] ^ key[xorIndex]);
      }

      return result;
   }

   public static String toHexString(byte[] pb) {
      StringBuffer sb = new StringBuffer();

      for(int i = 0; i < pb.length; ++i) {
         int val = pb[i] & 255;
         if(val < 16) {
            sb.append('0');
         }

         sb.append(Integer.toHexString(val));
      }

      return sb.toString();
   }

   public static byte[] toBytes(String hexString) {
      int size = hexString.length();
      byte[] ret = new byte[size / 2];
      byte[] tmp = hexString.getBytes();

      for(int i = 0; i < size / 2; ++i) {
         ret[i] = uniteBytes(tmp[i * 2], tmp[i * 2 + 1]);
      }

      return ret;
   }

   public static byte[] base64Decoder(String s) throws IOException {
      if(!StringUtils.isBlank(s) && !"eJwDAAAAAAE=".equalsIgnoreCase(s)) {
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         InflaterOutputStream zos = new InflaterOutputStream(bos);
         zos.write((new BASE64Decoder()).decodeBuffer(s));
         zos.close();
         return bos.toByteArray();
      } else {
         return null;
      }
   }

   public static String encryptHMAC(String type, byte[] content, byte[] keys) throws Exception {
      SecretKeySpec secretKey = new SecretKeySpec(keys, type);
      Mac mac = Mac.getInstance(secretKey.getAlgorithm());
      mac.init(secretKey);
      return toHexString(mac.doFinal(content));
   }

   private static byte uniteBytes(byte src0, byte src1) {
      char _b0 = (char)Byte.decode("0x" + new String(new byte[]{src0})).byteValue();
      _b0 = (char)(_b0 << 4);
      char _b1 = (char)Byte.decode("0x" + new String(new byte[]{src1})).byteValue();
      byte ret = (byte)(_b0 ^ _b1);
      return ret;
   }

   private static byte[] initKey(String aKey) {
      byte[] b_key = aKey.getBytes();
      byte[] state = new byte[256];

      int index1;
      for(index1 = 0; index1 < 256; ++index1) {
         state[index1] = (byte)index1;
      }

      index1 = 0;
      int index2 = 0;
      if(b_key != null && b_key.length != 0) {
         for(int i = 0; i < 256; ++i) {
            index2 = (b_key[index1] & 255) + (state[i] & 255) + index2 & 255;
            byte tmp = state[i];
            state[i] = state[index2];
            state[index2] = tmp;
            index1 = (index1 + 1) % b_key.length;
         }

         return state;
      } else {
         return null;
      }
   }

   private static Key getKey(byte[] keyBytes, String type) {
      int len = "des".equalsIgnoreCase(type)?8:16;
      byte[] array = new byte[len];
      System.arraycopy(keyBytes, 0, array, 0, Math.min(keyBytes.length, len));
      SecretKeySpec key = new SecretKeySpec(array, type);
      return key;
   }

   public static void main(String[] args) throws Exception {
      System.out.println("pwd: " + digestSha1("1qazcde3!@#", "sha-1"));
      System.out.println("pwd-shanshi: " + digestSha1("Hillstone@2020", "sha-1"));
   }

   public static class SM4 {

      static {
         Throwable e = null;
         String[] var4;
         int var3 = (var4 = new String[]{"64", "32"}).length;
         int var2 = 0;

         while(var2 < var3) {
            String w = var4[var2];

            try {
               String e1 = (new File(Constant.CONF_DIR, "sm4/" + w)).getAbsolutePath();
               String oldVal = System.getProperty("java.library.path");
               System.setProperty("java.library.path", e1 + File.pathSeparator + oldVal);
               Field sysPath = ClassLoader.class.getDeclaredField("sys_paths");
               sysPath.setAccessible(true);
               sysPath.set((Object)null, (Object)null);
               sysPath.setAccessible(false);
               System.loadLibrary("scm_sm4javasdk_dll_mar");
               e = null;
               break;
            } catch (Throwable var8) {
               e = var8;
               ++var2;
            }
         }

         if(e != null) {
            SecretUtil.LOG.error("加载sm4库出错", e);
         }

      }

      public static byte[] encryption(byte[] content, byte[] password) {
         content = SM4JSDK.addDataPadding(content);
         SM4JSDK sdk = new SM4JSDK();
         if(!sdk.createContext(password)) {
            throw new RuntimeException("初始化加密环境失败");
         } else {
            byte[] encContent = sdk.encryptBinary(content);
            if(encContent == null) {
               throw new RuntimeException("加密失败");
            } else {
               return encContent;
            }
         }
      }

      public static byte[] decode(byte[] content, byte[] password) {
         SM4JSDK sdk = new SM4JSDK();
         if(!sdk.createContext(password)) {
            throw new RuntimeException("初始化加密环境失败");
         } else {
            byte[] encContent = sdk.decryptBinary(content);
            if(encContent == null) {
               throw new RuntimeException("解密失败");
            } else {
               return SM4JSDK.removeDataPadding(encContent);
            }
         }
      }
   }
}
